﻿using CommunityToolkit.Mvvm.ComponentModel;
using CommunityToolkit.Mvvm.Input;
using LifeLogApp.Classes;
using LifeLogApp.Interfaces;
using LifeLogApp.Model;
using LifeLogApp.Services;
using Newtonsoft.Json.Linq;
using RestSharp;
using RestSharp.Extensions;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Reflection;
using System.Runtime.CompilerServices;
using System.Text;
using System.Threading.Tasks;

namespace LifeLogApp.ViewModel
{
    // Binding: https://www.youtube.com/watch?v=5Qga2pniN78&t=2s
    // https://devblogs.microsoft.com/dotnet/announcing-the-dotnet-community-toolkit-800/

    public partial class MainViewModel : ObservableObject
    {
        Serilog.ILogger logger;
        GeoService geoService;

        public Task taskAppSettingsLoop;
        public CancellationToken cancellationTokenAppSettingsLoop;

        public MainViewModel(Serilog.ILogger logger, GeoService geoService, IAppDeviceInfo getDeviceInfo)
        {
            this.logger = logger;
            this.geoService = geoService;

            // Set static properties
            AppVersion = AppInfo.Current.VersionString;
            DeviceID = getDeviceInfo.GetDeviceID();

            // Init properties
            CapturedEvents = new ObservableCollection<LifeLogEvent>();

            DoUpload = true;
            if (string.IsNullOrWhiteSpace(ApiKey))
            {
                Login(User, "PointUni2018!");
            }

            // Start AppSettingsLoop
            taskAppSettingsLoop = Task.Run(() => AppSettingsLoop(), cancellationTokenAppSettingsLoop);

            // Load LifeLog Events
            LoadCapturedEventsFromFiles(true).GetAwaiter();
        }

        public string User
        {
            get => Preferences.Get(nameof(User), null);
            set
            {
                var oldValue = Preferences.Get(nameof(User), null);
                if (string.Compare(value, oldValue, true) != 0)
                {
                    OnPropertyChanging(nameof(User));
                    Preferences.Set(nameof(User), value.Trim());
                    OnPropertyChanged(nameof(User));
                }
            }
        }

        public string ApiKey
        {
            get => Preferences.Get(nameof(ApiKey), null);
            set
            {
                var oldValue = Preferences.Get(nameof(ApiKey), null);
                if (string.Compare(value, oldValue, true) != 0)
                {
                    OnPropertyChanging(nameof(ApiKey));
                    Preferences.Set(nameof(ApiKey), value.Trim());
                    OnPropertyChanged(nameof(ApiKey));
                }
            }
        }

        public bool DoUpload
        {
            get => Preferences.Get(nameof(DoUpload), false);
            set
            {
                var oldValue = Preferences.Get(nameof(DoUpload), false);
                if (value != oldValue)
                {
                    OnPropertyChanging(nameof(DoUpload));
                    Preferences.Set(nameof(DoUpload), value);
                    OnPropertyChanged(nameof(DoUpload));
                }
            }
        }

        public string ShellStartForm
        {
            get => Preferences.Get(nameof(ShellStartForm), nameof(AppShell));
            set
            {
                var oldValue = Preferences.Get(nameof(ShellStartForm), nameof(AppShell));
                if (string.Compare(value, oldValue, true) != 0)
                {
                    OnPropertyChanging(nameof(ShellStartForm));
                    Preferences.Set(nameof(ShellStartForm), value.Trim());
                    OnPropertyChanged(nameof(ShellStartForm));
                }
            }
        }

        public LifeLogProjects ProjectList
        {
            get
            {
                string json = Preferences.Get(nameof(ProjectList), "{}");
                return Newtonsoft.Json.JsonConvert.DeserializeObject<LifeLogProjects>(json);
            }
            set
            {
                string json = Newtonsoft.Json.JsonConvert.SerializeObject(value, Newtonsoft.Json.Formatting.None);
                var oldJson = Preferences.Get(nameof(ProjectList), "{}");
                if (string.Compare(json, oldJson, true) != 0)
                {
                    OnPropertyChanging(nameof(ProjectList));
                    Preferences.Set(nameof(ProjectList), json);
                    OnPropertyChanged(nameof(ProjectList));
                }
            }
        }


        [ObservableProperty]
        string appVersion;

        [ObservableProperty]
        string deviceID;

        [ObservableProperty]
        string networkAccess;

        [ObservableProperty]
        bool permissionCamera;

        [ObservableProperty]
        bool permissionMicrophone;

        [ObservableProperty]
        bool permissionStorageRead;

        [ObservableProperty]
        bool permissionStorageWrite;

        [ObservableProperty]
        bool permissionLocationWhenInUse;

        [ObservableProperty]
        ObservableCollection<LifeLogEvent> capturedEvents;

        public void RaiseOnPropertyChangedUploadFiles()
        {
            OnPropertyChanged(nameof(UploadFiles));
        }
        public ObservableCollection<string> UploadFiles
        {
            get
            {
                var uploadFiles = new ObservableCollection<string>();
                foreach (var item in Directory.GetFiles(AppHelper.AppPathEventsUpload, "*.*"))
                {
                    uploadFiles.Add(Path.GetFileName(item));
                }
                return uploadFiles;
            }
        }

        [RelayCommand]
        async void RequestPermissions()
        {
            await MainThread.InvokeOnMainThreadAsync(async () => await AppHelper.RequestAllPermissionsAsync());
        }

        [RelayCommand]
        async void ReLoadCapturedEventsFromFiles()
        {
            await LoadCapturedEventsFromFiles(false);
        }

        [RelayCommand]
        async void DebugClearAndRefresh()
        {
            await LoadCapturedEventsFromFiles(true);
        }

        [RelayCommand]
        async void DebugClearFileSystemAppDataDirectory()
        {
            await Task.Run(() =>
            {
                foreach (var item in Directory.GetDirectories(FileSystem.AppDataDirectory))
                {
                    Directory.Delete(item, true);
                }
                foreach (var item in Directory.GetFiles(FileSystem.AppDataDirectory))
                {
                    File.Delete(item);
                }
            });
            RaiseOnPropertyChangedUploadFiles();
            CapturedEvents = new ObservableCollection<LifeLogEvent>();
            SaveCapturedEventsToSettings();
        }

        [RelayCommand]
        void Settings() => AppInfo.Current.ShowSettingsUI();

        [RelayCommand]
        void ShellStartFormSetTab()
        {
            ShellStartForm = nameof(AppShell);
        }

        [RelayCommand]
        void ShellStartFormSetFlyout()
        {
            ShellStartForm = nameof(AppShell2);
        }

        [RelayCommand]
        void Tap(Guid id)
        {
            var tapedEvent = CapturedEvents.First(x => x.LifeLogEventId == id);
            MainThread.BeginInvokeOnMainThread(
                async () =>
                {
                    await Application.Current.MainPage
                        .DisplayAlert("Tap", $"You did tap {tapedEvent.FileName}", "OK");
                });

            //await Shell.Current.GoToAsync($"{nameof(DetailPage)}?Text={s}");
        }

        [RelayCommand]
        async void TakePhoto() => await TakePhotoAsync();

        [RelayCommand]
        async void TakeVideo() => await TakeVideoAsync();

        [RelayCommand]
        async void UploadCapturedEvents()
        {
            await Shell.Current.GoToAsync(nameof(UploadPage));
        }

        [RelayCommand]
        void DeleteCapturedEventFile(string eventFile)
        {
            if (File.Exists(eventFile))
            {
                File.Delete(eventFile);
            }
            var lifelogEvent = capturedEvents.FirstOrDefault(x => x.LocalMediaFullFileName == eventFile);
            if (capturedEvents != null)
            {
                capturedEvents.Remove(lifelogEvent);
                SaveCapturedEventsToSettings();
            }

            ReLoadCapturedEventsFromFiles();
        }


        public async void AppSettingsLoop()
        {
            logger.Verbose("{Method}: Start", AppHelper.BuildFullMethodName(MethodBase.GetCurrentMethod()));

            while (!cancellationTokenAppSettingsLoop.IsCancellationRequested)
            {
                NetworkAccess = Connectivity.Current.NetworkAccess.ToString();

                PermissionCamera = await Permissions.CheckStatusAsync<Permissions.Camera>() == PermissionStatus.Granted;

                PermissionMicrophone = await Permissions.CheckStatusAsync<Permissions.Microphone>() == PermissionStatus.Granted;

                PermissionStorageRead = await Permissions.CheckStatusAsync<Permissions.StorageRead>() == PermissionStatus.Granted;

                PermissionStorageWrite = await Permissions.CheckStatusAsync<Permissions.StorageWrite>() == PermissionStatus.Granted;

                PermissionLocationWhenInUse = await Permissions.CheckStatusAsync<Permissions.LocationWhenInUse>() == PermissionStatus.Granted;

                await Task.Delay(5000);
            }
        }


        public async Task LoadCapturedEventsFromFiles(bool clearAndLoad)
        {
            if (clearAndLoad)
            {
                capturedEvents.Clear();

                SaveCapturedEventsToSettings();
            }

            // Load LifeLogEvents from Settings
            LoadCapturedEventsFromSettings();

            // Get all local event files
            var files = Directory.GetFiles(AppHelper.AppPathEventsCapture, "*.*", SearchOption.AllDirectories).OrderByDescending(x => new FileInfo(x).CreationTime).ToList();

            // Shorten list
            foreach (var item in capturedEvents)
            {
                if (files.Contains(item.LocalMediaFullFileName))
                {
                    files.Remove(item.LocalMediaFullFileName);
                }
            }

            // Add new files
            foreach (var f in files)
            {
                var currentLocation = await geoService.GetCurrentLocationAsync();
                var reverseGeoCodeData = await geoService.GetReverseGeocodeDataAsync(currentLocation.Latitude, currentLocation.Longitude);
                var formattedAddress = geoService.BuildFormattedAddress(reverseGeoCodeData.FirstOrDefault());

                var lifelogEvent = new LifeLogEvent()
                {
                    LifeLogEventId = Guid.Parse(Path.GetFileNameWithoutExtension(f)),
                    WhenOrTimeUTC = new FileInfo(f).CreationTime.ToUniversalTime(),
                    WhereLocation = currentLocation,
                    WherePlace = formattedAddress,
                    WhoOrUserIdOrEmail = User,
                    LocalMediaFullFileName = f,
                };
                capturedEvents.Add(lifelogEvent);
            }

            // Save LifeLogEvents to Settings 
            SaveCapturedEventsToSettings();
        }

        public void LoadCapturedEventsFromSettings()
        {
            var json = Preferences.Get("CapturedEvents", null);
            if (json == null)
            {
                CapturedEvents = new ObservableCollection<LifeLogEvent>();
                return;
            }

            CapturedEvents = Newtonsoft.Json.JsonConvert.DeserializeObject<ObservableCollection<LifeLogEvent>>(json);
        }

        public void SaveCapturedEventsToSettings()
        {
            var json = Newtonsoft.Json.JsonConvert.SerializeObject(capturedEvents, Newtonsoft.Json.Formatting.Indented);

            Preferences.Set("CapturedEvents", json);
        }


        async Task TakePhotoAsync()
        {
            await MainThread.InvokeOnMainThreadAsync(async () =>
            {
                if (!MediaPicker.Default.IsCaptureSupported)
                {
                    logger.Error("No Camera");
                    await Application.Current.MainPage.DisplayAlert("Error", "No Camera", "Ok");
                    return;
                }

                await Permissions.RequestAsync<Permissions.Camera>();
                await Permissions.RequestAsync<Permissions.StorageRead>();
                await Permissions.RequestAsync<Permissions.StorageWrite>();
                await Permissions.RequestAsync<Permissions.LocationWhenInUse>();

                var photoFile = await MediaHelper.TakePhotoAsync();
                if (photoFile != null)
                {
                    logger.Debug("Photo {File}", photoFile);
                }
                else
                {
                    logger.Error("No Photo was taken");
                    await Application.Current.MainPage.DisplayAlert("Error", "No Photo was taken", "Ok");
                }
            });

            ReLoadCapturedEventsFromFiles();
        }

        async Task TakeVideoAsync()
        {
            await MainThread.InvokeOnMainThreadAsync(async () =>
            {
                if (!MediaPicker.Default.IsCaptureSupported)
                {
                    logger.Error("No Camera");
                    await Application.Current.MainPage.DisplayAlert("Error", "No Camera", "Ok");
                }
                else
                {
                    var videoFile = await MediaHelper.TakeVideoAsync();
                    if (videoFile != null)
                    {
                        logger.Debug("Video {File}", videoFile);
                    }
                    else
                    {
                        logger.Error("No Video was taken");
                        await Application.Current.MainPage.DisplayAlert("Error", "No Video was taken", "Ok");
                    }
                }
            });

            ReLoadCapturedEventsFromFiles();
        }

        public bool Login(string email, string password)
        {
            var client = new RestClient(
                $"https://my.unipoint.se/LifeLog_Dev/api/user/{email}/login/{password}");
            client.Timeout = -1;
            var request = new RestRequest(Method.GET);

            var fullUrl = client.BuildUri(request);
            logger.Verbose("FullUrl={FullUrl}", fullUrl);

            IRestResponse response = client.Execute(request);
            logger.Verbose("Response={StatusCode}, Content={Content}, ErrorMessage={ErrorMessage}",
                response.StatusCode, response.Content, response.ErrorMessage);

            if (response.IsSuccessful)
            {
                logger.Information(
                    "{Method}: Login Successful {User}",
                    AppHelper.BuildFullMethodName(MethodBase.GetCurrentMethod()),
                    User);

                var jobj = JObject.Parse(response.Content);
                ApiKey = jobj.GetValue("ApiKey").Value<string>();

                return true;
            }

            return false;

        }


    }
}
