﻿using System;
using System.Collections.ObjectModel;
using System.Reflection;
using CommunityToolkit.Mvvm.ComponentModel;
using CommunityToolkit.Mvvm.Input;
using LifeLogApp.Classes;
using LifeLogApp.Model;
using RestSharp;

namespace LifeLogApp.ViewModel
{
    public partial class UploadViewModel : ObservableObject
    {
        Serilog.ILogger logger;
        MainViewModel mainViewModel;

        public UploadViewModel(Serilog.ILogger logger, MainViewModel mainViewModel)
        {
            this.mainViewModel = mainViewModel;

            ProjectOrFolderList = new ObservableCollection<string>();
            var pl = mainViewModel.ProjectList;

            var client = new RestClient("https://my.unipoint.se/LifeLog_Dev/api/project/list");
            client.Timeout = -1;
            var request = new RestRequest(Method.GET);
            request.AddHeader("ApiKey", mainViewModel.ApiKey);

            var fullUrl = client.BuildUri(request);
            logger.Verbose("FullUrl={FullUrl}", fullUrl);

            IRestResponse response = client.Execute(request);
            logger.Verbose("Response={StatusCode}, Content={Content}, ErrorMessage={ErrorMessage}",
                response.StatusCode, response.Content, response.ErrorMessage);

            if (response.IsSuccessful)
            {
                logger.Information(
                    "{Method}: Projets downloaded OK",
                    AppHelper.BuildFullMethodName(MethodBase.GetCurrentMethod()));

                var projectList = Newtonsoft.Json.JsonConvert.DeserializeObject<LifeLogProjects>(response.Content);
                mainViewModel.ProjectList = projectList;
                foreach (var p in projectList.Projects)
                {
                    ProjectOrFolderList.Add(p.FolderName);
                }
            }
            else
            {
                logger.Error("Response={StatusCode}, Content={Content}, ErrorMessage={ErrorMessage}",
                    response.StatusCode, response.Content, response.ErrorMessage);
            }
        }

        [ObservableProperty]
        ObservableCollection<string> projectOrFolderList;

        public string UploadSelectedProjectOrFolder
        {
            get => Preferences.Get(nameof(UploadSelectedProjectOrFolder), null);
            set
            {
                var oldValue = Preferences.Get(nameof(UploadSelectedProjectOrFolder), null);
                if (string.Compare(value, oldValue, true) != 0)
                {
                    OnPropertyChanging(nameof(UploadSelectedProjectOrFolder));
                    Preferences.Set(nameof(UploadSelectedProjectOrFolder), value);
                    OnPropertyChanged(nameof(UploadSelectedProjectOrFolder));
                }
            }
        }

        [ObservableProperty]
        string tag;

        public string UploadWhat
        {
            get => Preferences.Get(nameof(UploadWhat), null);
            set
            {
                var oldValue = Preferences.Get(nameof(UploadWhat), null);
                if (string.Compare(value, oldValue, true) != 0)
                {
                    OnPropertyChanging(nameof(UploadWhat));
                    Preferences.Set(nameof(UploadWhat), value);
                    OnPropertyChanged(nameof(UploadWhat));
                }
            }
        }

        [RelayCommand]
        async void Upload()
        {
            var capturedEvents = mainViewModel.CapturedEvents.ToList();

            foreach (var item in capturedEvents)
            {
                item.FolderOrProject = mainViewModel.ProjectList.Projects.First(x => x.FolderName == UploadSelectedProjectOrFolder).MoveToFolder;
                //item.Tags = new List<string>() { Tag };
                item.WhatOrHowOrWhyOrEventOrAction = UploadWhat;

                var json = Newtonsoft.Json.JsonConvert.SerializeObject(item, Newtonsoft.Json.Formatting.Indented);
                var jsonFileName = Path.Combine(AppHelper.AppPathEventsUpload, Path.GetFileNameWithoutExtension(item.LocalMediaFullFileName) + ".json");
                await File.WriteAllTextAsync(jsonFileName, json);

                var newUploadFileName = Path.Combine(AppHelper.AppPathEventsUpload, Path.GetFileName(item.LocalMediaFullFileName));
                File.Move(item.LocalMediaFullFileName, newUploadFileName);

                mainViewModel.CapturedEvents.Remove(item);
            }

            mainViewModel.SaveCapturedEventsToSettings();
            mainViewModel.LoadCapturedEventsFromSettings();

            mainViewModel.RaiseOnPropertyChangedUploadFiles();

            await Shell.Current.GoToAsync("..");
        }

    }
}

